@@ -24,14 +24,15 @@ from releng_tool.engine.fetch import stage as fetch_stage
 from releng_tool.engine.init import initialize_sample
 from releng_tool.exceptions import RelengToolMissingConfigurationError
 from releng_tool.exceptions import RelengToolMissingPackagesError
+from releng_tool.opts import RELENG_POST_BUILD_NAME
 from releng_tool.packages.exceptions import RelengToolStageFailure
 from releng_tool.packages.manager import RelengPackageManager
 from releng_tool.packages.pipeline import RelengPackagePipeline
 from releng_tool.prerequisites import RelengPrerequisites
 from releng_tool.registry import RelengRegistry
 from releng_tool.stats import RelengStats
-from releng_tool.util.env import extend_script_env
 from releng_tool.util.env import env_value
+from releng_tool.util.env import extend_script_env
 from releng_tool.util.file_flags import FileFlag
 from releng_tool.util.file_flags import check_file_flag
 from releng_tool.util.file_flags import process_file_flag
@@ -109,7 +110,7 @@ class RelengEngine:
             'releng_version': releng_version,
         }
 
-        verbose('loading statistics...')
+        debug('loading statistics...')
         self.stats.load()
 
         # prepare script environment to make helpers available to configuration
@@ -217,9 +218,12 @@ class RelengEngine:
                 PkgAction.PATCH,
             ]
 
+            # determine if an explicit fetch request has been made
+            requested_fetch = None
+            if gaction == GlobalAction.FETCH or pa == PkgAction.FETCH:
+                requested_fetch = True
+
             # ensure all package sources are acquired first
-            requested_fetch = (
-                gaction == GlobalAction.FETCH or pa == PkgAction.FETCH)
             for pkg in pkgs:
                 if not self._stage_init(pkg):
                     return False
@@ -421,8 +425,12 @@ of the releng process:
             path_remove(self.opts.out_dir)
 
             verbose('removing file flags')
-            path_remove(self.opts.ff_local_srcs)
-            path_remove(self.opts.ff_devmode)
+            if os.path.exists(self.opts.ff_devmode):
+                if path_remove(self.opts.ff_devmode):
+                    warn('development mode has been unconfigured')
+            if os.path.exists(self.opts.ff_local_srcs):
+                if path_remove(self.opts.ff_local_srcs):
+                    warn('local-sources mode has been unconfigured')
         else:
             verbose('removing build directory')
             path_remove(self.opts.build_dir)
@@ -509,28 +517,36 @@ of the releng process:
             post-processing script exists; ``False`` if an error has occurred
             when processing the post-processing script
         """
-        script, script_exists = opt_file(self.opts.post_point)
+        build_script, postbuild_exists = opt_file(self.opts.post_build_point)
 
         # TODO remove deprecated configuration load in (at maximum) v1.0
-        if not script_exists:
-            script = os.path.join(self.opts.root_dir, 'post.py')
-            if os.path.isfile(script):
-                warn('using deprecated post-processing file post.py -- switch '
-                     'to releng-post for future projects')
-                script_exists = True
+        if not postbuild_exists:
+            deprecated_posts = [
+                'releng-post',
+                'post.py',
+            ]
 
-        if script_exists:
-            verbose('performing post-processing...')
+            for script_name in deprecated_posts:
+                build_script = os.path.join(self.opts.root_dir, script_name)
+                if os.path.isfile(build_script):
+                    warn('using deprecated post-processing file {} -- switch '
+                         'to {} for future projects', script_name,
+                         RELENG_POST_BUILD_NAME)
+                    postbuild_exists = True
+                    break
+
+        if postbuild_exists:
+            verbose('performing post-processing (build)...')
 
             # ensure images directory exists (as the post-processing script will
             # most likely populate it)
             if not ensure_dir_exists(self.opts.images_dir):
                 return False
 
-            if not run_script(script, env, subject='post-processing'):
+            if not run_script(build_script, env, subject='post-build'):
                 return False
 
-            verbose('post-processing completed')
+            verbose('post-processing (build) completed')
 
         return True
 
