@@ -23,14 +23,25 @@ class GitExistsType(Enum):
     revision exists in a Git repository.
 
     Attributes:
-        EXISTS: revision exists
+        EXISTS_BRANCH: branch exists
+        EXISTS_HASH: hash exists
+        EXISTS_TAG: tag exists
         MISSING: revision does not exist
         MISSING_HASH: a hash-provided revision does not exist
     """
-    EXISTS = 'exists'
+    EXISTS_BRANCH = 'exists_branch'
+    EXISTS_HASH = 'exists_hash'
+    EXISTS_TAG = 'exists_tag'
     MISSING = 'missing'
     MISSING_HASH = 'missing_hash'
 
+# types indicating a revision exists
+REVISION_EXISTS = [
+    GitExistsType.EXISTS_BRANCH,
+    GitExistsType.EXISTS_HASH,
+    GitExistsType.EXISTS_TAG,
+]
+
 def fetch(opts):
     """
     support fetching from git sources
@@ -58,45 +69,28 @@ def fetch(opts):
 
     # check if we have the target revision cached; if so, package is ready
     if os.path.isdir(cache_dir) and not opts.ignore_cache:
-        if revision_exists(git_dir, revision) == GitExistsType.EXISTS:
+        erv = revision_exists(git_dir, revision)
+        if erv in REVISION_EXISTS:
             # ensure configuration is properly synchronized
             if not _sync_git_configuration(opts):
                 return None
 
-            return cache_dir
+            # if no explicit ignore-cache request and if the revision is a
+            # branch, force ignore-cache on and allow fetching to proceed
+            if opts.ignore_cache is None and erv == GitExistsType.EXISTS_BRANCH:
+                opts.ignore_cache = True
+            # return cache dir if not verifying or verification succeeds
+            elif not opts._git_verify_revision or _verify_revision(
+                    git_dir, revision, quiet=True):
+                return cache_dir
 
     note('fetching {}...'.format(name))
     sys.stdout.flush()
 
-    # if we have a cache dir, ensure it's stable
-    #
-    # If we have a cache directory for this page but didn't find the the target
-    # revision above, first check if the Git cache has been corrupted. If
-    # anything is suspected wrong, start from a fresh state.
-    has_cache = False
-    clean_cache = False
-    if os.path.isdir(cache_dir):
-        if opts.ignore_cache:
-            verbose('sanity checking if cached directory is valid')
-            if GIT.execute([git_dir, 'rev-parse'], cwd=cache_dir, quiet=True):
-                has_cache = True
-            else:
-                clean_cache = True
-        else:
-            log('cache directory exists for package; validating')
-            if GIT.execute([git_dir, 'fsck', '--full'], cwd=cache_dir,
-                    quiet=True):
-                has_cache = True
-            else:
-                clean_cache = True
-
-    if clean_cache:
-        log('cache directory has errors; will re-downloaded')
-
-        if not path_remove(cache_dir):
-            err('''unable to cleanup cache folder for package
-(cache folder: {})'''.format(cache_dir))
-            return None
+    # validate any cache directory (if one exists)
+    has_cache, bad_validation = _validate_cache(cache_dir)
+    if bad_validation:
+        return None
 
     # if we have no cache for this repository, build one
     if not has_cache:
@@ -115,6 +109,20 @@ def fetch(opts):
     if not _fetch_srcs(opts, cache_dir, revision, refspecs=opts._git_refspecs):
         return None
 
+    # verify revision (if configured to check it)
+    if opts._git_verify_revision:
+        if not _verify_revision(git_dir, revision):
+            err("""\
+failed to validate git revision
+
+Package has been configured to require the verification of the GPG signature
+for the target revision. The verification has failed. Ensure that the revision
+is signed and that the package's public key has been registered in the system.
+
+      Package: {}
+     Revision: {}""".format(name, revision))
+            return None
+
     # fetch submodules (if configured to do so)
     if opts._git_submodules:
         if not _fetch_submodules(opts, cache_dir, revision):
@@ -229,7 +237,7 @@ def _fetch_srcs(opts, cache_dir, revision, desc=None, refspecs=None):
     if revision:
         verbose('verifying target revision exists')
         exists_state = revision_exists(git_dir, revision)
-        if exists_state == GitExistsType.EXISTS:
+        if exists_state in REVISION_EXISTS:
             pass
         elif (exists_state == GitExistsType.MISSING_HASH and
                 limited_fetch and opts._git_depth is None):
@@ -242,7 +250,7 @@ def _fetch_srcs(opts, cache_dir, revision, desc=None, refspecs=None):
                 err('unable to unshallow fetch state')
                 return False
 
-            if revision_exists(git_dir, revision) != GitExistsType.EXISTS:
+            if revision_exists(git_dir, revision) not in REVISION_EXISTS:
                 err('unable to find matching revision in ' + desc)
                 err(' (revision: {}) '.format(revision))
                 return False
@@ -268,6 +276,10 @@ def revision_exists(git_dir, revision):
         a value of ``GitExistsType``
     """
 
+    if GIT.execute([git_dir, 'rev-parse', '--quiet', '--verify',
+            'refs/tags/' + revision], quiet=True):
+        return GitExistsType.EXISTS_TAG
+
     output = []
     if not GIT.execute([git_dir, 'rev-parse', '--quiet', '--verify',
             revision], quiet=True, capture=output):
@@ -283,10 +295,12 @@ def revision_exists(git_dir, revision):
     # valid provided. If so, perform a `cat-file` request to ensure the long
     # hash entry is indeed a valid commit.
     if output and output[0] == revision:
-        if not GIT.execute([git_dir, 'cat-file', '-t', revision], quiet=True):
+        if GIT.execute([git_dir, 'cat-file', '-t', revision], quiet=True):
+            return GitExistsType.EXISTS_HASH
+        else:
             return GitExistsType.MISSING_HASH
 
-    return GitExistsType.EXISTS
+    return GitExistsType.EXISTS_BRANCH
 
 def _sync_git_configuration(opts):
     """
@@ -471,33 +485,16 @@ def _fetch_submodule(opts, name, cache_dir, revision, site):
         if not revision:
             return _sync_git_origin(cache_dir, site)
 
-        if revision_exists(git_dir, revision) == GitExistsType.EXISTS:
+        if revision_exists(git_dir, revision) in REVISION_EXISTS:
             return _sync_git_origin(cache_dir, site)
 
     log('processing submodule (package: {}) {}...'.format(opts.name, name))
     sys.stdout.flush()
 
-    # if we have a cache dir, ensure it's stable
-    #
-    # If we have a cache directory for this page but didn't find the the target
-    # revision above, first check if the Git cache has been corrupted. If
-    # anything is suspected wrong, start from a fresh state.
-    has_cache = False
-    if os.path.isdir(cache_dir):
-        if opts.ignore_cache:
-            has_cache = True
-        else:
-            log('cache directory exists for submodule; validating')
-            if GIT.execute([git_dir, 'fsck', '--full'], cwd=cache_dir,
-                    quiet=True):
-                has_cache = True
-            else:
-                log('cache directory has errors; will re-downloaded')
-
-                if not path_remove(cache_dir):
-                    err('''unable to cleanup cache folder for package
- (cache folder: {})'''.format(cache_dir))
-                    return False
+    # validate any cache directory (if one exists)
+    has_cache, bad_validation = _validate_cache(cache_dir)
+    if bad_validation:
+        return None
 
     # if we have no cache for this repository, build one
     if not has_cache:
@@ -515,3 +512,73 @@ def _fetch_submodule(opts, name, cache_dir, revision, site):
     # fetch sources for this submodule
     desc = 'submodule ({}): {}'.format(opts.name, name)
     return _fetch_srcs(opts, cache_dir, revision, desc=desc)
+
+def _validate_cache(cache_dir):
+    """
+    validate an existing cache directory to fetch on
+
+    A fetch operation may occur on an existing cache directory, typically when
+    a force-fetch or a configured revision has changed. This call helps
+    validate the existing cache directory (from a bad state such as a corrupted
+    repository). If a cache directory does exist,
+
+    Args:
+        cache_dir: the cache/bare repository to fetch into
+
+    Returns:
+        a 2-tuple (if a cache directory exists; and if validation failed)
+    """
+
+    git_dir = '--git-dir=' + cache_dir
+
+    bad_validation = False
+    has_cache = False
+    if os.path.isdir(cache_dir):
+        log('cache directory detected; validating')
+        if GIT.execute([git_dir, 'rev-parse'], cwd=cache_dir, quiet=True):
+            debug('cache directory validated')
+            has_cache = True
+        else:
+            log('cache directory has errors; will re-downloaded')
+            if not path_remove(cache_dir):
+                err('unable to cleanup cache folder for package\n'
+                    ' (cache folder: {})', cache_dir)
+                bad_validation = True
+
+    return has_cache, bad_validation
+
+def _verify_revision(git_dir, revision, quiet=False):
+    """
+    verify the gpg signature for a target revision
+
+    The GPG signature for a provided revision (tag or commit) will be checked
+    to validate the revision.
+
+    Args:
+        git_dir: the Git directory
+        revision: the revision to verify
+        quiet (optional): whether or not the log if verification is happening
+
+    Returns:
+        ``True`` if the revision is signed; ``False`` otherwise
+    """
+
+    if not quiet:
+        log('verifying the gpg signature on the target revision')
+    else:
+        verbose('verifying the gpg signature on the target revision')
+
+    if GIT.execute([git_dir, 'rev-parse', '--quiet',
+            '--verify', revision + '^{tag}'], quiet=True):
+        verified_cmd = 'verify-tag'
+    else:
+        verified_cmd = 'verify-commit'
+
+        # acquire the commit if (if not already set), to ensure we can verify
+        # against commits or branches
+        rv, revision = GIT.execute_rv(git_dir, 'rev-parse', revision)
+        if rv != 0:
+            verbose('failed to determine the commit id for a revision')
+            return False
+
+    return GIT.execute([git_dir, verified_cmd, revision], quiet=quiet)
