@@ -23,6 +23,7 @@ from releng_tool.defs import RPK_GIT_CONFIG
 from releng_tool.defs import RPK_GIT_DEPTH
 from releng_tool.defs import RPK_GIT_REFSPECS
 from releng_tool.defs import RPK_GIT_SUBMODULES
+from releng_tool.defs import RPK_GIT_VERIFY_REVISION
 from releng_tool.defs import RPK_INSTALL_DEFS
 from releng_tool.defs import RPK_INSTALL_ENV
 from releng_tool.defs import RPK_INSTALL_OPTS
@@ -142,6 +143,7 @@ class RelengPackageManager:
         self._register_conf(RPK_GIT_DEPTH, PkgKeyType.INT_NONNEGATIVE)
         self._register_conf(RPK_GIT_REFSPECS, PkgKeyType.STRS)
         self._register_conf(RPK_GIT_SUBMODULES, PkgKeyType.BOOL)
+        self._register_conf(RPK_GIT_VERIFY_REVISION, PkgKeyType.BOOL)
         self._register_conf(RPK_INSTALL_DEFS, PkgKeyType.DICT_STR_STR)
         self._register_conf(RPK_INSTALL_ENV, PkgKeyType.DICT_STR_STR)
         self._register_conf(RPK_INSTALL_OPTS, PkgKeyType.DICT_STR_STR_OR_STRS)
@@ -244,7 +246,7 @@ class RelengPackageManager:
         sorted_pkgs = []
         for pkg in pkgs.values():
             sorted_pkgs = sorter.sort(pkg)
-            if not sorted:
+            if sorted_pkgs is None:
                 raise RelengToolCyclicPackageDependency({
                     'pkg_name': name,
                 })
@@ -489,6 +491,9 @@ class RelengPackageManager:
         # git-submodules
         pkg_git_submodules = self._fetch(RPK_GIT_SUBMODULES)
 
+        # git-verify
+        pkg_git_verify_revision = self._fetch(RPK_GIT_VERIFY_REVISION)
+
         # ######################################################################
 
         # checks
@@ -639,6 +644,7 @@ class RelengPackageManager:
 
         # (commons)
         pkg = RelengPackage(name, pkg_version)
+        pkg.asc_file = os.path.join(pkg_def_dir, name + '.asc')
         pkg.build_dir = pkg_build_dir
         pkg.build_output_dir = pkg_build_output_dir
         pkg.build_subdir = pkg_build_subdir
@@ -651,6 +657,7 @@ class RelengPackageManager:
         pkg.git_depth = pkg_git_depth
         pkg.git_refspecs = pkg_git_refspecs
         pkg.git_submodules = pkg_git_submodules
+        pkg.git_verify_revision = pkg_git_verify_revision
         pkg.has_devmode_option = pkg_has_devmode_option
         pkg.hash_file = os.path.join(pkg_def_dir, name + '.hash')
         pkg.is_internal = pkg_is_internal
